@@ -110,11 +110,15 @@ typedef enum {
 	opt_add_samba_data,
 	opt_samba_data_tool,
 	opt_trusted_for_delegation,
+	opt_dont_expire_password,
 	opt_add_service_principal,
 	opt_remove_service_principal,
 	opt_description,
+	opt_setattr,
+	opt_delattr,
 	opt_use_ldaps,
 	opt_account_disable,
+	opt_ldap_passwd,
 } Option;
 
 static adcli_tool_desc common_usages[] = {
@@ -144,11 +148,15 @@ static adcli_tool_desc common_usages[] = {
 	{ opt_computer_password_lifetime, "lifetime of the host accounts password in days", },
 	{ opt_trusted_for_delegation, "set/unset the TRUSTED_FOR_DELEGATION flag\n"
 	                              "in the userAccountControl attribute", },
+	{ opt_dont_expire_password, "set/unset the DONT_EXPIRE_PASSWORD flag\n"
+	                            "in the userAccountControl attribute", },
 	{ opt_account_disable, "set/unset the ACCOUNTDISABLE flag\n"
 	                       "in the userAccountControl attribute", },
 	{ opt_add_service_principal, "add the given service principal to the account\n" },
 	{ opt_remove_service_principal, "remove the given service principal from the account\n" },
 	{ opt_description, "add a description to the account\n" },
+	{ opt_setattr, "add an attribute with a value\n" },
+	{ opt_delattr, "remove an attribute\n" },
 	{ opt_no_password, "don't prompt for or read a password" },
 	{ opt_prompt_password, "prompt for a password if necessary" },
 	{ opt_stdin_password, "read a password from stdin (until EOF) if\n"
@@ -162,6 +170,7 @@ static adcli_tool_desc common_usages[] = {
 	{ opt_add_samba_data, "add domain SID and computer account password\n"
 	                      "to the Samba specific configuration database" },
 	{ opt_samba_data_tool, "Absolute path to the tool used for add-samba-data" },
+	{ opt_ldap_passwd, "Use LDAP add/mod operation to set/change password" },
 	{ opt_verbose, "show verbose progress and failure messages", },
 	{ 0 },
 };
@@ -307,6 +316,13 @@ parse_option (Option opt,
 			adcli_enroll_set_trusted_for_delegation (enroll, false);
 		}
 		return ADCLI_SUCCESS;
+	case opt_dont_expire_password:
+		if (strcasecmp (optarg, "true") == 0 || strcasecmp (optarg, "yes") == 0) {
+			adcli_enroll_set_dont_expire_password (enroll, true);
+		} else {
+			adcli_enroll_set_dont_expire_password (enroll, false);
+		}
+		return ADCLI_SUCCESS;
 	case opt_account_disable:
 		if (strcasecmp (optarg, "true") == 0 || strcasecmp (optarg, "yes") == 0) {
 			adcli_enroll_set_account_disable (enroll, true);
@@ -323,6 +339,18 @@ parse_option (Option opt,
 	case opt_description:
 		adcli_enroll_set_description (enroll, optarg);
 		return ADCLI_SUCCESS;
+	case opt_setattr:
+		ret =  adcli_enroll_add_setattr (enroll, optarg);
+		if (ret != ADCLI_SUCCESS) {
+			warnx ("parsing setattr option failed");
+		}
+		return ret;
+	case opt_delattr:
+		ret = adcli_enroll_add_delattr (enroll, optarg);
+		if (ret != ADCLI_SUCCESS) {
+			warnx ("parsing delattr option failed");
+		}
+		return ret;
 	case opt_use_ldaps:
 		adcli_conn_set_use_ldaps (conn, true);
 		return ADCLI_SUCCESS;
@@ -334,6 +362,7 @@ parse_option (Option opt,
 	case opt_show_password:
 	case opt_one_time_password:
 	case opt_add_samba_data:
+	case opt_ldap_passwd:
 		assert (0 && "not reached");
 		break;
 	}
@@ -391,13 +420,16 @@ adcli_tool_computer_join (adcli_conn *conn,
 		{ "os-version", required_argument, NULL, opt_os_version },
 		{ "os-service-pack", optional_argument, NULL, opt_os_service_pack },
 		{ "description", optional_argument, NULL, opt_description },
+		{ "setattr", required_argument, NULL, opt_setattr },
 		{ "user-principal", optional_argument, NULL, opt_user_principal },
 		{ "trusted-for-delegation", required_argument, NULL, opt_trusted_for_delegation },
+		{ "dont-expire-password", required_argument, NULL, opt_dont_expire_password },
 		{ "add-service-principal", required_argument, NULL, opt_add_service_principal },
 		{ "show-details", no_argument, NULL, opt_show_details },
 		{ "show-password", no_argument, NULL, opt_show_password },
 		{ "add-samba-data", no_argument, NULL, opt_add_samba_data },
 		{ "samba-data-tool", required_argument, 0, opt_samba_data_tool },
+		{ "ldap-passwd", no_argument, NULL, opt_ldap_passwd },
 		{ "verbose", no_argument, NULL, opt_verbose },
 		{ "help", no_argument, NULL, 'h' },
 		{ 0 },
@@ -429,6 +461,9 @@ adcli_tool_computer_join (adcli_conn *conn,
 		case opt_add_samba_data:
 			flags |= ADCLI_ENROLL_ADD_SAMBA_DATA;
 			break;
+		case opt_ldap_passwd:
+			flags |= ADCLI_ENROLL_LDAP_PASSWD;
+			break;
 		case 'h':
 		case '?':
 		case ':':
@@ -513,9 +548,12 @@ adcli_tool_computer_update (adcli_conn *conn,
 		{ "os-version", required_argument, NULL, opt_os_version },
 		{ "os-service-pack", optional_argument, NULL, opt_os_service_pack },
 		{ "description", optional_argument, NULL, opt_description },
+		{ "setattr", required_argument, NULL, opt_setattr },
+		{ "delattr", required_argument, NULL, opt_delattr },
 		{ "user-principal", optional_argument, NULL, opt_user_principal },
 		{ "computer-password-lifetime", optional_argument, NULL, opt_computer_password_lifetime },
 		{ "trusted-for-delegation", required_argument, NULL, opt_trusted_for_delegation },
+		{ "dont-expire-password", required_argument, NULL, opt_dont_expire_password },
 		{ "account-disable", required_argument, NULL, opt_account_disable },
 		{ "add-service-principal", required_argument, NULL, opt_add_service_principal },
 		{ "remove-service-principal", required_argument, NULL, opt_remove_service_principal },
@@ -523,6 +561,7 @@ adcli_tool_computer_update (adcli_conn *conn,
 		{ "show-password", no_argument, NULL, opt_show_password },
 		{ "add-samba-data", no_argument, NULL, opt_add_samba_data },
 		{ "samba-data-tool", required_argument, 0, opt_samba_data_tool },
+		{ "ldap-passwd", no_argument, NULL, opt_ldap_passwd },
 		{ "verbose", no_argument, NULL, opt_verbose },
 		{ "help", no_argument, NULL, 'h' },
 		{ 0 },
@@ -550,6 +589,9 @@ adcli_tool_computer_update (adcli_conn *conn,
 		case opt_add_samba_data:
 			flags |= ADCLI_ENROLL_ADD_SAMBA_DATA;
 			break;
+		case opt_ldap_passwd:
+			flags |= ADCLI_ENROLL_LDAP_PASSWD;
+			break;
 		case 'h':
 		case '?':
 		case ':':
@@ -885,7 +927,7 @@ adcli_tool_computer_reset (adcli_conn *conn,
 	parse_fqdn_or_name (enroll, argv[0]);
 	adcli_enroll_reset_computer_password (enroll);
 
-	res = adcli_enroll_password (enroll, 0);
+	res = adcli_enroll_password (enroll);
 	if (res != ADCLI_SUCCESS) {
 		warnx ("resetting %s in %s domain failed: %s", argv[0],
 		       adcli_conn_get_domain_name (conn),
